// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.bankbalance;

import java.math.BigDecimal;

import net.sf.chex4j.Contract;
import net.sf.chex4j.Pre;

@Contract
public class PublicBankAccountImplWithChex implements IBankAccount, ITaxable {

	/**
	 * This field is public to allow you to corrupt the invariants of the class. 
	 * Clearly if you want to protect the invariants of a class you need to 
	 * encapsulate its contents. 
	 */
	public BigDecimal balance = new BigDecimal(0);

	/* (non-Javadoc)
	 * @see net.sf.chex4j.bankbalance.IBankAccount#getBalance()
	 */
	public BigDecimal getBalance(){
		return this.balance;
	}

	/* (non-Javadoc)
	 * @see net.sf.chex4j.bankbalance.IBankAccount#deposit(java.math.BigDecimal)
	 */
	public BigDecimal deposit(BigDecimal amount) {
		this.balance = this.balance.add(amount);
		return this.balance;
	}

	/* (non-Javadoc)
	 * @see net.sf.chex4j.bankbalance.IBankAccount#withdraw(java.math.BigDecimal)
	 */
	public BigDecimal withdraw(BigDecimal amount){
		this.balance = this.balance.subtract(amount);
		return this.balance;
	}

	@Pre("amount.doubleValue() >= 0.0d")
	public void setBalanceDirectly(BigDecimal amount) {
		this.balance = amount;
	}

	/* (non-Javadoc)
	 * @see net.sf.chex4j.bankbalance.ITaxable#tax(int)
	 */
	public BigDecimal tax(int percent) {
		BigDecimal multipler = (new BigDecimal(percent)).divide(new BigDecimal(100));
		BigDecimal tax = this.balance.multiply(multipler);
		this.balance = this.balance.subtract(tax);
		return this.balance;
	}


}
