// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.bankbalance;

import java.math.BigDecimal;

import net.sf.chex4j.Contract;
import net.sf.chex4j.Post;
import net.sf.chex4j.Pre;

@Contract
public class SimplePublicBankAccount {

	@Post(message = "Initial balance should be greater than or equal to zero.", value = "amount.doubleValue() >= 0.0d")
	public SimplePublicBankAccount(BigDecimal amount){
		this.balance = amount;
	}
	
	/**
	 * This field is public to allow you to corrupt the invariants of the class.
	 * Clearly if you want to protect the invariants of a class you need to
	 * encapsulate its contents.
	 */
	public BigDecimal balance = new BigDecimal(0);

	@Post(message = "Balance should be greater than zero.", value = "$_.doubleValue() >= 0.0d")
	public BigDecimal getBalance() {
		return this.balance;
	}

	@Pre("amount.doubleValue() >= 0.0d")
	@Post("$_.doubleValue() == this.balance.doubleValue()")
	public BigDecimal deposit(BigDecimal amount) {
		this.balance = this.balance.add(amount);
		return this.balance;
	}

	@Pre("amount.doubleValue() >= 0.0d")
	@Post("$_.doubleValue() >= 0.0d")
	public BigDecimal withdraw(BigDecimal amount) {
		this.balance = this.balance.subtract(amount);
		return this.balance;
	}

}
