// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.bankbalance.tests;

import static org.junit.Assert.assertTrue;

import java.math.BigDecimal;

import net.sf.chex4j.bankbalance.OverloadAdditionalChexBankAccount;
import net.sf.chex4j.bankbalance.OverloadNoAdditionalChexBankAccount;

import org.junit.Before;
import org.junit.Test;

public class TestMethodOverloading {
	private OverloadNoAdditionalChexBankAccount bankAccountNoAdditional;
	private OverloadAdditionalChexBankAccount bankAccountWithAdditional;

	@Before
	public void setUp() throws Exception {
		BigDecimal bd = new BigDecimal(0);
		bankAccountNoAdditional = new OverloadNoAdditionalChexBankAccount(bd );
		bankAccountWithAdditional = new OverloadAdditionalChexBankAccount(bd);
	}

	@Test
	public void worksWithoutTriggeringChexs() throws Exception {
		BigDecimal bd = new BigDecimal("50");
		bankAccountNoAdditional.balance = bd;
		bankAccountWithAdditional.balance = bd;
		assertTrue(!bankAccountWithAdditional.methodCalled);
		assertTrue(!bankAccountNoAdditional.methodCalled);
		bankAccountWithAdditional.withdraw(new BigDecimal("5"));
		bankAccountNoAdditional.withdraw(new BigDecimal("1"));
		assertTrue(bankAccountWithAdditional.methodCalled);
		assertTrue(bankAccountNoAdditional.methodCalled);
	}

	@Test(expected = AssertionError.class)
	public void worksNoAdditionalSuperPreChexFires() throws Exception {
		BigDecimal bd = new BigDecimal("50");
		bankAccountNoAdditional.balance = bd;
		assertTrue(!bankAccountNoAdditional.methodCalled);
		bankAccountNoAdditional.withdraw(new BigDecimal("-1"));
		assertTrue(bankAccountNoAdditional.methodCalled);
	}
	
	@Test(expected = AssertionError.class)
	public void worksYesAdditionalSuperPreChexFires() throws Exception {
		BigDecimal bd = new BigDecimal("50");
		bankAccountWithAdditional.balance = bd;
		assertTrue(!bankAccountWithAdditional.methodCalled);
		bankAccountWithAdditional.withdraw(new BigDecimal("-1"));
		assertTrue(bankAccountWithAdditional.methodCalled);
	}

	@Test(expected = AssertionError.class)
	public void worksNoAdditionalSuperPostChexFires() throws Exception {
		BigDecimal bd = new BigDecimal("-50");
		bankAccountNoAdditional.balance = bd;
		bankAccountNoAdditional.withdraw(new BigDecimal("1"));
	}
	
	@Test(expected = AssertionError.class)
	public void worksYesAdditionalSuperPostChexFires() throws Exception {
		BigDecimal bd = new BigDecimal("-50");
		bankAccountWithAdditional.balance = bd;
		bankAccountWithAdditional.withdraw(new BigDecimal("1"));
	}

	@Test(expected = AssertionError.class)
	public void worksYesAdditionalSubPreChexFires() throws Exception {
		BigDecimal bd = new BigDecimal("50");
		bankAccountWithAdditional.balance = bd;
		bankAccountWithAdditional.withdraw(new BigDecimal("0.5"));
	}

	@Test(expected = AssertionError.class)
	public void worksYesAdditionalSubPostChexFires() throws Exception {
		BigDecimal bd = new BigDecimal("5");
		bankAccountWithAdditional.balance = bd;
		bankAccountWithAdditional.withdraw(new BigDecimal("0.5"));
	}
}
