// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.bankbalance.tests;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.math.BigDecimal;

import org.junit.Before;
import org.junit.Test;

import net.sf.chex4j.bankbalance.PrivateLogicChexBankAccount;

public class TestPrivateLogicChex {
	private PrivateLogicChexBankAccount privateLogicChexBankAccount = null;
	
	@Before
	public void setUp() throws Exception {
		BigDecimal bd = new BigDecimal(0);
		this.privateLogicChexBankAccount = new PrivateLogicChexBankAccount(bd);
	}
	
	@Test
	public void testPrelogic() throws Exception {
		assertThat(this.privateLogicChexBankAccount.preLogicCalled, is(false));
		this.privateLogicChexBankAccount.balance = new BigDecimal(0.5d);
		boolean threwError = false;
		try {
			this.privateLogicChexBankAccount.withdraw(new BigDecimal(0.1d));
		} catch ( AssertionError e){
			threwError = true;
		}
		assertTrue(threwError);
		assertThat(this.privateLogicChexBankAccount.preLogicCalled, is(true));
	}
	
	@Test
	public void testPostLogic() throws Exception {
		assertThat(this.privateLogicChexBankAccount.postLogicCalled, is(false));
		this.privateLogicChexBankAccount.balance = new BigDecimal(6.0d);
		boolean threwError = false;
		try {
			this.privateLogicChexBankAccount.withdraw(new BigDecimal(4.0d));
		} catch ( AssertionError e){
			threwError = true;
		}
		assertTrue(threwError);
		assertThat(this.privateLogicChexBankAccount.postLogicCalled, is(true));
	}

	@Test
	public void testHappyPath() throws Exception {
		assertThat(this.privateLogicChexBankAccount.postLogicCalled, is(false));
		assertThat(this.privateLogicChexBankAccount.preLogicCalled, is(false));
		
		this.privateLogicChexBankAccount.balance = new BigDecimal(8.0d);
		this.privateLogicChexBankAccount.withdraw(new BigDecimal(1.1d));
		
		assertThat(this.privateLogicChexBankAccount.postLogicCalled, is(true));
		assertThat(this.privateLogicChexBankAccount.preLogicCalled, is(true));
	}

}
