// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.bankbalance.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.math.BigDecimal;

import net.sf.chex4j.bankbalance.IBankAccount;
import net.sf.chex4j.bankbalance.ITaxable;
import net.sf.chex4j.bankbalance.PublicBankAccountImplWithoutChex;

import org.junit.Before;
import org.junit.Test;

/**
 * All tests in this class will fail unless you run with VM argument -javaagent:target/chex4j-0.0.1-SNAPSHOT.jar
 * @author simon
 *
 */
public class TestPublicInterfaceImplHasNoChex {
	private IBankAccount bankAccount;
	
	@Before
	public void setUp() throws Exception {
		bankAccount = new PublicBankAccountImplWithoutChex();
	}

	@Test
	public void checkImplIsChex() throws Exception {
		((PublicBankAccountImplWithoutChex)bankAccount).setBalanceDirectly(new BigDecimal(-1));
		return;
	}
	
	@Test
	public void bankAccountCanAcceptPostiveDeposit() throws Exception {
		BigDecimal b = bankAccount.deposit(new BigDecimal(1.1f));
		assertEquals( b, bankAccount.getBalance());
		return;
	}

	@Test
	public void bankAccountCanTaxPostiveAmount() throws Exception {
		BigDecimal b = bankAccount.deposit(new BigDecimal(1.0d));
		assertEquals( b, bankAccount.getBalance());
		ITaxable taxable = (ITaxable) bankAccount;
		taxable.tax(23);
		double balance = bankAccount.getBalance().doubleValue();
		double remainder = balance - 0.77d;
		assertTrue( Math.abs(remainder) < 0.00001d );
		return;
	}

	
	@Test( expected = java.lang.AssertionError.class )
	public void bankAccountCannotAcceptNegativeDeposit() throws Exception {
		BigDecimal b = bankAccount.deposit(new BigDecimal(-1));
		assertEquals( b, bankAccount.getBalance());
		return;
	}
	
	@Test
	public void bankAccountWillReturnPostiveBalance() throws Exception {
		bankAccount.deposit(new BigDecimal(1));
		BigDecimal b = bankAccount.getBalance();
		assertEquals( new BigDecimal(1), b);
		return;
	}

	@Test( expected = java.lang.AssertionError.class )
	public void bankAccountWillNotReturnNegativeBalance() throws Exception {
		((PublicBankAccountImplWithoutChex)bankAccount).balance = new BigDecimal(-1);
		BigDecimal b = bankAccount.getBalance();
		assertEquals( new BigDecimal(-1), b);
		return;
	}

	@Test( expected = java.lang.AssertionError.class )
	public void otherPackageBankAccountWillNotReturnNegativeBalance() throws Exception {
		IBankAccount bankAccount = new net.sf.chex4j.other.PublicBankAccountImplWithoutChex(); 
		((net.sf.chex4j.other.PublicBankAccountImplWithoutChex)bankAccount).balance = new BigDecimal(-1);
		BigDecimal b = bankAccount.getBalance();
		assertEquals( new BigDecimal(-1), b);
		return;
	}
	
	@Test
	public void bankAccountCanWithdrawPostiveAmount() throws Exception {
		bankAccount.deposit(new BigDecimal(10));
		bankAccount.withdraw(new BigDecimal(1));
		BigDecimal b = bankAccount.getBalance();
		assertEquals( new BigDecimal(9), b);
		return;
	}
	
	@Test( expected = java.lang.AssertionError.class )
	public void bankAccountCannotWithdrawNegativeAmount() throws Exception {
		bankAccount.deposit(new BigDecimal(10));
		bankAccount.withdraw(new BigDecimal(-1));
		BigDecimal b = bankAccount.getBalance();
		assertEquals( new BigDecimal(11), b);
	}

	@Test( expected = java.lang.AssertionError.class )
	public void bankAccountCanotGoOverdrawn() throws Exception {
		bankAccount.deposit(new BigDecimal(10));
		bankAccount.withdraw(new BigDecimal(11));
		BigDecimal b = bankAccount.getBalance();
		assertEquals( new BigDecimal(-1), b);
	}
}
