// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.bankbalance.tests;

import static org.hamcrest.Matchers.is;
import static org.hamcrest.Matchers.containsString;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.math.BigDecimal;

import net.sf.chex4j.bankbalance.RegularMethodsBankAccount;

import org.junit.Before;
import org.junit.Test;

public class TestRegularMethodChex {
	private RegularMethodsBankAccount regularMethodsBankAccount = null;
	
	@Before
	public void setUp() throws Exception {
		BigDecimal bd = new BigDecimal(0);
		this.regularMethodsBankAccount = new RegularMethodsBankAccount(bd);
	}
	
	@Test
	public void testPrelogic() throws Exception {
		assertThat(this.regularMethodsBankAccount.preLogicCalled, is(false));
		this.regularMethodsBankAccount.balance = new BigDecimal(0.5d);
		boolean threwError = false;
		try {
			this.regularMethodsBankAccount.withdraw(new BigDecimal(0.1d));
		} catch ( AssertionError e){
			assertThat(e.toString(), containsString("This bank account requires all withdraws to be greater than 1.0"));
			threwError = true;
		}
		assertTrue(threwError);
		assertThat(this.regularMethodsBankAccount.preLogicCalled, is(true));
	}
	
	@Test
	public void testPostLogic() throws Exception {
		assertThat(this.regularMethodsBankAccount.postLogicCalled, is(false));
		this.regularMethodsBankAccount.balance = new BigDecimal(6.0d);
		boolean threwError = false;
		try {
			this.regularMethodsBankAccount.withdraw(new BigDecimal(4.0d));
		} catch ( AssertionError e){
			assertThat(e.toString(), containsString("This bank account requires a minimum balance of 5.0"));
			threwError = true;
		}
		assertTrue(threwError);
		assertThat(this.regularMethodsBankAccount.postLogicCalled, is(true));
	}
	
	@Test
	public void testHappyPath() throws Exception {
		assertThat(this.regularMethodsBankAccount.postLogicCalled, is(false));
		assertThat(this.regularMethodsBankAccount.preLogicCalled, is(false));
		
		this.regularMethodsBankAccount.balance = new BigDecimal(8.0d);
		this.regularMethodsBankAccount.withdraw(new BigDecimal(1.1d));
		
		assertThat(this.regularMethodsBankAccount.postLogicCalled, is(true));
		assertThat(this.regularMethodsBankAccount.preLogicCalled, is(true));
	}
	
	@Test( expected = AssertionError.class)
	public void testBaseClassConstructor() throws Exception {
		new RegularMethodsBankAccount(new BigDecimal(-1.1f));
	}

}
