// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.bankbalance.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.math.BigDecimal;

import net.sf.chex4j.Post;
import net.sf.chex4j.Pre;
import net.sf.chex4j.bankbalance.SimplePublicBankAccount;

import org.junit.Before;
import org.junit.Test;

/**
 * All tests in this class will fail unless you run with VM argument -javaagent:target/chex4j-0.0.1-SNAPSHOT.jar
 * @author simon
 *
 */
public class TestSimplePublicClass {
	private SimplePublicBankAccount bankAccount;
	
	@Before
	public void setUp() throws Exception {
		bankAccount = new SimplePublicBankAccount(new BigDecimal(0.0f));
	}

	@Test( expected = AssertionError.class)
	public void constructorChex() throws Exception {
		new SimplePublicBankAccount(new BigDecimal(-1.1f));
		return;
	}

	
	@Test
	public void bankAccountCanAcceptPostiveDeposit() throws Exception {
		BigDecimal b = bankAccount.deposit(new BigDecimal(1.1f));
		assertEquals( b, bankAccount.balance);
		return;
	}
	
	@Test( expected = java.lang.AssertionError.class )
	public void bankAccountCannotAcceptNegativeDeposit() throws Exception {
		BigDecimal b = bankAccount.deposit(new BigDecimal(-1));
		assertEquals( b, bankAccount.balance);
		return;
	}
	
	@Test
	public void bankAccountWillReturnPostiveBalance() throws Exception {
		bankAccount.deposit(new BigDecimal(1));
		BigDecimal b = bankAccount.getBalance();
		assertEquals( new BigDecimal(1), b);
		return;
	}

	@Test( expected = java.lang.AssertionError.class )
	public void bankAccountWillNotReturnNegativeBalance() throws Exception {
		bankAccount.balance = new BigDecimal(-1);
		BigDecimal b = bankAccount.getBalance();
		assertEquals( new BigDecimal(-1), b);
		return;
	}
	
	@Test
	public void bankAccountCanWithdrawPostiveAmount() throws Exception {
		bankAccount.deposit(new BigDecimal(10));
		bankAccount.withdraw(new BigDecimal(1));
		BigDecimal b = bankAccount.getBalance();
		assertEquals( new BigDecimal(9), b);
		return;
	}
	
	@Test( expected = java.lang.AssertionError.class )
	public void bankAccountCannotWithdrawNegativeAmount() throws Exception {
		bankAccount.deposit(new BigDecimal(10));
		bankAccount.withdraw(new BigDecimal(-1));
		BigDecimal b = bankAccount.getBalance();
		assertEquals( new BigDecimal(11), b);
	}

	@Test( expected = java.lang.AssertionError.class )
	public void bankAccountCannotGoOverdrawn() throws Exception {
		bankAccount.deposit(new BigDecimal(10));
		bankAccount.withdraw(new BigDecimal(11));
		BigDecimal b = bankAccount.getBalance();
		assertEquals( new BigDecimal(-1), b);
	}

	@Test
	public void bankAccountPostMessageTest() throws Exception {
		bankAccount.balance = new BigDecimal(-1);
		try {
			bankAccount.getBalance();
			assertTrue(false);
		} catch (AssertionError e){
			Post post = bankAccount.getClass().getMethod("getBalance", (Class<?>[])null ).getAnnotation(Post.class);
			assertTrue( e.getMessage().contains(post.message()));
		}
		return;
	}

	@Test
	public void bankAccountPreMessageTest() throws Exception {
		try {
			bankAccount.withdraw(new BigDecimal(-1));
			assertTrue(false);
		} catch (AssertionError e){
			Pre pre = bankAccount.getClass().getMethod("withdraw", new Class<?>[]{BigDecimal.class}).getAnnotation(Pre.class);
			assertTrue( e.getMessage().contains(pre.message()));
		}
		return;
	}	
}
